import 'dart:collection';
import 'dart:convert';

import 'package:enum_to_string/enum_to_string.dart';
import 'package:objd/basic/condition.dart';
import 'package:objd/basic/entity.dart';
import 'package:objd/basic/for_list.dart';
import 'package:objd/basic/score.dart';
import 'package:objd/basic/scoreboard.dart';
import 'package:objd/basic/widget.dart';
import 'package:objd/wrappers/execute.dart';

class Detect {
	static final HashMap<DetectType, Scoreboard> _boards = HashMap();
	static final HashMap<Entity, Score> _scores = HashMap();

	static Widget load() {
		return For.of([
			..._boards.values
		]);
	}

	static Widget tick() {
		List<Widget> actions = [];

		_scores.forEach((k, v) {
			actions.add(Execute(as: k, If: Condition.score(v.matchesRange(Range(from: 1))), children: [v.set(0)]));
		});

		return For.of([
			...actions
		]);
	}

	static String _parseScoreboardName(DetectType event) {
		const String prefix = 'ct_';
		final String asString = EnumToString.parse(event);

		if (asString.length > (16 - prefix.length)) {
			var bytes = utf8.encode(asString);
			var base64Str = base64.encode(bytes);
			var shortened = base64Str.substring(0, (16 - prefix.length));
			return '${prefix}${shortened}';
		}

		return '${prefix}${asString}';
	}

	static Score _addDetect(Entity entity, DetectType event) {
		final String scoreboardName = _parseScoreboardName(event);
		final String eventType = 'minecraft.custom:minecraft.${EnumToString.parse(event)}';

		if (!_boards.containsKey(event)) {
			_boards[event] = Scoreboard(scoreboardName, type: eventType, addIntoLoad: false);
		}

		if (!_scores.containsKey(entity)) {
			_scores[entity] = Score(entity, scoreboardName, addNew: false);
		}

		return _scores[entity];
	}

	static Condition when(DetectType event, [Entity entity]) {
		final entityOrPlayer = entity ?? Entity.Player();
		return Condition.score(_addDetect(entityOrPlayer, event).matchesRange(Range(from: 1)));
	}
}

enum DetectType {
	animals_bred,
	aviate_one_cm,
	bell_ring,
	boat_one_cm,
	clean_armor,
	clean_banner,
	clean_shulker_box,
	climb_one_cm,
	crouch_one_cm,
	damage_absorbed,
	damage_blocked_by_shield,
	damage_dealt,
	damage_dealt_absorbed,
	damage_dealt_resisted,
	damage_resisted,
	damage_taken,
	deaths,
	drop,
	eat_slice_cake,
	enchant_item,
	fall_one_cm,
	fill_cauldron,
	fish_caught,
	fly_one_cm,
	horse_one_cm,
	inspect_dispenser,
	inspect_dropper,
	inspect_hopper,
	interact_with_beacon,
	interact_with_blast_furnace,
	interact_with_brewingstand,
	interact_with_campfire,
	interact_with_cartography_table,
	interact_with_crafting_table,
	interact_with_furnace,
	interact_with_lectern,
	interact_with_loom,
	interact_with_smoker,
	interact_with_stone_cutter,
	jump,
	leave_game,
	minecart_one_cm,
	mob_kills,
	open_barrel,
	open_chest,
	open_enderchest,
	open_shulker_box,
	pig_one_cm,
	play_noteblock,
	play_one_minute,
	play_record,
	player_kills,
	pot_flower,
	raid_trigger,
	raid_win,
	sleep_in_bed,
	sneak_time,
	sprint_one_cm,
	swim_one_cm,
	talked_to_villager,
	time_since_death,
	time_since_rest,
	traded_with_villager,
	trigger_trapped_chest,
	tune_noteblock,
	use_cauldron,
	walk_on_water_one_cm,
	walk_one_cm,
	walk_under_water_one_cm
}
