import 'dart:collection';

import 'package:objd/basic/condition.dart';
import 'package:objd/basic/entity.dart';
import 'package:objd/basic/for_list.dart';
import 'package:objd/basic/score.dart';
import 'package:objd/basic/scoreboard.dart';
import 'package:objd/basic/widget.dart';

class Stat {
	static final HashMap<String, Scoreboard> _boards = HashMap();
	static final HashMap<String, HashMap<Entity, Score>> _scores = HashMap();

	static Widget load() {
		return For.of([
			..._boards.values,
			..._scores.values.fold([], (k, v) => [ ...k, ...v.values])
		]);
	}

	static Widget tick() {
		return null;
	}

	static Score _addStat(String name, Entity entity) {
		if (!_boards.containsKey(name)) {
			_boards[name] = Scoreboard(name, addIntoLoad: false);
		}

		if (!_scores.containsKey(name)) {
			_scores[name] = HashMap();
		}

		if (!_scores[name].containsKey(entity)) {
			_scores[name][entity] = Score(entity, name, addNew: false);
		}

		return _scores[name][entity];
	}

	static Condition when(String name, Score Function(Score) action, [Entity entity]) {
		return Condition.score(then(name, action, entity));
	}

	static Score then(String name, Score Function(Score) action, [Entity entity]) {
		final entityOrPlayer = entity ?? Entity.Player();
		return action(_addStat(name, entityOrPlayer));
	}

	static Score set(String name, int value, [Entity entity]) {
		return then(name, (Score s) => s.set(value), entity);
	}

	static Score add(String name, int value, [Entity entity]) {
		return then(name, (Score s) => s..add(value), entity);
	}

	static Score subtract(String name, int value, [Entity entity]) {
		return then(name, (Score s) => s.subtract(value), entity);
	}
}
